using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Threading;

namespace CustomScrollableControl
{
	/// <summary>
	/// Summary description for CustomScrollableControl.
	/// </summary>
	public class CustomScrollableControl : System.Windows.Forms.UserControl
	{
        // Designer generated.....
		private System.Windows.Forms.HScrollBar hScrollBar;
		private System.Windows.Forms.VScrollBar vScrollBar;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
        
        /// <summary>
        /// The bitmap of the virtual space.
        /// </summary>
        private Image scrollingImage;
        /// <summary>
        /// Accessor for the scrollImage Property.
        /// The set calls the adjustScrollBars method
        /// in order to adjust the scroll bars when a
        /// new image is loaded. 
        /// </summary>
        public Image ScrollingImage
        {
            get
            {
                return scrollingImage;
            }
            set
            {
                scrollingImage = value;
                adjustScrollBars();
            }
        }
        
        /// <summary>
        /// The Coordinates of the viewport.
        /// </summary>
        private Point viewportCoords;
        
        /// <summary>
        /// Returns the actual Viewprot Coordinates.
        /// Used by applications to be able to determine 
        /// the image coordinates relatice to the image.
        /// </summary>
        public Point ViewportCoords()
        {
            return viewportCoords;
        }

        /// <summary>
        /// Returns a rectangle the size from the
        /// origin of the control to the x-Coordinate
        /// of the vertical scrollbar and the y-Coordinate
        /// of the horizontal scrollbar.
        /// </summary>
        public Rectangle ScrollingImageArea
        {
            get
            {
                return new Rectangle(0, 0, this.vScrollBar.Left, this.hScrollBar.Top);
            }
            set
            {
            }
        }

		public CustomScrollableControl()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitComponent call
			this.SetStyle(ControlStyles.Opaque, true);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.hScrollBar = new System.Windows.Forms.HScrollBar();
			this.vScrollBar = new System.Windows.Forms.VScrollBar();
			this.SuspendLayout();
			// 
			// hScrollBar
			// 
			this.hScrollBar.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.hScrollBar.Location = new System.Drawing.Point(0, 133);
			this.hScrollBar.Name = "hScrollBar";
			this.hScrollBar.Size = new System.Drawing.Size(150, 17);
			this.hScrollBar.TabIndex = 0;
			this.hScrollBar.Scroll += new System.Windows.Forms.ScrollEventHandler(this.hScrollBar_Scroll);
			// 
			// vScrollBar
			// 
			this.vScrollBar.Dock = System.Windows.Forms.DockStyle.Right;
			this.vScrollBar.Location = new System.Drawing.Point(133, 0);
			this.vScrollBar.Name = "vScrollBar";
			this.vScrollBar.Size = new System.Drawing.Size(17, 133);
			this.vScrollBar.TabIndex = 1;
			this.vScrollBar.Value = 3;
			this.vScrollBar.Scroll += new System.Windows.Forms.ScrollEventHandler(this.vScrollBar_Scroll);
			// 
			// CustomScrollableControl
			// 
			this.Controls.Add(this.vScrollBar);
			this.Controls.Add(this.hScrollBar);
			this.Name = "CustomScrollableControl";
			this.Resize += new System.EventHandler(this.CustomScrollableControl_Resize);
			this.Load += new System.EventHandler(this.CustomScrollableControl_Load);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.CustomScrollableControl_Paint);
			this.ResumeLayout(false);

		}
		#endregion

		

		/// <summary>
		/// Calculates and draws the part of the bitmap that is visible to the control
		/// using a clip rectangle.
		/// </summary>
		private void drawImage(Graphics graphics, int xPosition, int yPosition)
		{
			graphics.SetClip(ScrollingImageArea);
			Rectangle drawingArea = new Rectangle(new Point(xPosition, yPosition), ScrollingImageArea.Size);
			graphics.DrawImage(this.scrollingImage, ScrollingImageArea, drawingArea, GraphicsUnit.Pixel);
		
		}
		/// <summary>
		/// Paint event handler for CustomScrolableControl.
		/// It takes care of copyiong the necessary image date to the screnn and
		/// whites out any other area to the right or below the image that needs
		/// repainting.
		/// </summary>
		private void CustomScrollableControl_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			Graphics graphics = e.Graphics;
			Brush solidBrush = new SolidBrush(this.BackColor);

			if (this.scrollingImage != null)
			{
				drawImage(graphics, this.viewportCoords.X, this.viewportCoords.Y);
				graphics.ResetClip();

				Rectangle rect;

				// Whiten out the area right to image and left of scroll bar
				rect = new Rectangle(scrollingImage.Width, 0, 
					this.vScrollBar.Left - scrollingImage.Width, 
					this.ScrollingImageArea.Height);
				if (! rect.IsEmpty)
				{
					graphics.FillRectangle(solidBrush, rect);
				}

				// Whiten area below image and above horizontal scroll bar
				rect = new Rectangle(0, scrollingImage.Height, 
					this.ScrollingImageArea.Width,
					this.hScrollBar.Top - scrollingImage.Height);
				if (! rect.IsEmpty)
				{
					graphics.FillRectangle(solidBrush, rect);
				}
				
				// Fill the small bottom right rectangle 
				rect = new Rectangle(this.hScrollBar.Right, this.vScrollBar.Bottom,
					this.ClientRectangle.Width - this.hScrollBar.Right,
					this.ClientRectangle.Height - this.vScrollBar.Bottom);
				graphics.FillRectangle(solidBrush, rect);
			}
			else
			{
				graphics.FillRectangle(solidBrush, this.ClientRectangle);
			}
		}

        /// <summary>
        /// Dimensions and to sets some properties for the scrollbars
        /// </summary>
		private void adjustScrollBars()
		{
			const int divisor = 10;
			if (scrollingImage != null)
			{
                // Horizontal minimum set to 0
				hScrollBar.Minimum = 0;
                // maximum is the image width
				hScrollBar.Maximum = scrollingImage.Width;
                // Large chage
				hScrollBar.LargeChange = ScrollingImageArea.Width;
                //small change
				hScrollBar.SmallChange = ScrollingImageArea.Width / divisor;
				hScrollBar.Value = viewportCoords.X;

                // Vertical minimum is 0
				vScrollBar.Minimum = 0;
                // Vertical maximum is height of image
				vScrollBar.Maximum = scrollingImage.Height;
                // Large change
				vScrollBar.LargeChange = ScrollingImageArea.Height;
                // small change
				vScrollBar.SmallChange = ScrollingImageArea.Height / divisor;
				vScrollBar.Value = viewportCoords.Y;
			}
		}

		/// <summary>
		/// This is the heart of the custom control. In this method the viewport
		/// is incrementally moved from the previous psoition to the current position of the
		/// scrollbars. In between showing the incremental moves the methods slpees
		/// for a short time to display the image bfore moving to the next poition
		/// until the final destination.
		/// </summary>
		private void scroll(Point prevPoint, Point curPoint)
		{
			const int ms = 10;
			const int diffDiv = 15;
			Graphics graphics = this.CreateGraphics();
			if(prevPoint.X != curPoint.X)
			{
				// Horizontal scrolling
				bool scrollingUp = curPoint.X - prevPoint.X > 0;
				// If change to scroll bar small then set incremental diff to 1 or -1
				// depending on scroll direction
				int diff = (curPoint.X - prevPoint.X) / diffDiv;
				if(scrollingUp && diff < 1)
				{
					diff = 1;
				}
				if(!scrollingUp && diff > -1)
				{
					diff = -1;
				}
				int temp = prevPoint.X;

				// Show the incremental scroll effect by looping over the changes
				while(true)
				{
					if(temp == curPoint.X)
					{
						break;
					}
					temp += diff;
					// In case the increment was too much set the 
					// set the temp variable to the current scrollbar value
					if((scrollingUp && temp > curPoint.X) ||
						(!scrollingUp && temp < curPoint.X))
					{
						temp = curPoint.X;
						continue;
					}
					
					// Draws the part of the image taht is visible to the control
					drawImage(graphics, temp, prevPoint.Y);
					if(temp != curPoint.X)
					{
						Thread.Sleep(ms);
					}
				}
			}
			if (prevPoint.Y != curPoint.Y)
			{	
				// Vertical scrolling
				bool scrollingUp = curPoint.Y - prevPoint.Y > 0;
				int diff = (curPoint.Y - prevPoint.Y) / diffDiv;
				// If change to scroll bar small then set incremental diff to 1 or -1
				// depending on scroll direction
				if(scrollingUp && diff < 1)
				{
					diff = 1;
				}
				if(!scrollingUp && diff > -1)
				{
					diff = -1;
				}
				int temp = prevPoint.Y;
				// Show the incremental scroll effect by looping over the changes
				while(true)
				{
					if(temp == curPoint.Y)
					{
						break;
					}
					temp += diff;
					// In case the increment was too much set the 
					// set the temp variable to the current scrollbar value
					if ((scrollingUp && temp > curPoint.Y) ||
							(!scrollingUp && temp < curPoint.Y))
					{
						temp = curPoint.Y;
						continue;
					}
					
					// Draws the part of the image taht is visible to the control
					drawImage(graphics, prevPoint.X, temp);
					if(temp != curPoint.Y)
					{
						Thread.Sleep(ms);
					}
				}
			}
		}
		
		/// <summary>
		/// Called when the vertical scrollbar was clicked by the user.
		/// The new coordinates according to the current scrollbar value is
		/// calculated and the scroll method is called to do the smooth
		/// scrolling from the old viewport position to the new one.
		/// </summary>
		private void vScrollBar_Scroll(object sender, System.Windows.Forms.ScrollEventArgs e)
		{
			switch( e.Type)
			{
				// switching depending on type of scrolling performed
				case ScrollEventType.SmallIncrement:
				case ScrollEventType.LargeIncrement:
				case ScrollEventType.SmallDecrement:
				case ScrollEventType.LargeDecrement:
				// Store the viewportCoords as previous origin
					Point prevOrigin = viewportCoords;
					// Scrolling in vertical direction ( in Y direction), 
					// current position of the scrollbar set to new viewportCoords
					viewportCoords = new Point(viewportCoords.X, e.NewValue);
					// Calls the actual smooth scrolling method with the previous
					// and the curent origin of the viewport.
					scroll(prevOrigin, viewportCoords);
					break;
				default:
					viewportCoords = new Point(viewportCoords.X, e.NewValue);
					this.Invalidate();
					break;
			}
		}
		
		/// <summary>
		/// Called when the horizontal scrollbar was clicked by the user.
		/// The new coordinates according to the current scrollbar value is
		/// calculated and the scroll method is called to do the smooth
		/// scrolling from the old viewport position to the new one.
		/// </summary>
		private void hScrollBar_Scroll(object sender, System.Windows.Forms.ScrollEventArgs e)
		{
            #region Do It Yourself Chapter 5
			switch( e.Type)
			{
				// switching depending on type of scrolling performed
				case ScrollEventType.SmallIncrement:
				case ScrollEventType.LargeIncrement:
				case ScrollEventType.SmallDecrement:
				case ScrollEventType.LargeDecrement:
					// Store the viewportCoords as previous origin
					Point prevOrigin = viewportCoords;
					// Scrolling in horizontal direction ( in X direction), 
					// current position of the scrollbar set to new viewportCoords
					viewportCoords = new Point(e.NewValue, viewportCoords.Y);
					// Calls the actual smooth scrolling method with the previous
					// and the curent origin of the viewport.
					scroll(prevOrigin, viewportCoords);
					break;
				default:
					viewportCoords = new Point(e.NewValue, viewportCoords.Y);
					this.Invalidate();
					break;
			}
            #endregion
		}
		/// <summary>
		/// Called whenver the window is resized. The scrollbar positions
		/// are recalculated and the image coordinates for the repaint are
		/// calculated.
		/// </summary>
		private void CustomScrollableControl_Resize(object sender, System.EventArgs e)
		{
			if (scrollingImage != null)
			{
				// returns the smaller value of viewportCoords.X
				// or scrollingImage.Width - ScrollingImageArea.Width
				int x = Math.Min(viewportCoords.X,
					scrollingImage.Width - ScrollingImageArea.Width);
				// stes x to zero if the calculation before 
				// returned a negative value
				x = Math.Max(x, 0);
				// returns smaller value of the provided paramters.
				int y = Math.Min(viewportCoords.Y,
					scrollingImage.Height - ScrollingImageArea.Height);
				y = Math.Max(y, 0);
				// Sets the viewportCoords to new values
				viewportCoords = new Point(x, y);
			}
			// calls the method to adjust scrollbars
			adjustScrollBars();
			// invalidates the control to force a repaint after
			// resizing.
			this.Invalidate();
		}

        private void CustomScrollableControl_Load(object sender, System.EventArgs e)
        {
		
        }
		
	}
}
