using System;
using System.Drawing;
using NUnit.Framework;

namespace Photo_Editor_Application
{
	/// <summary>
	/// Summary description for UnitTest.
	/// </summary>
	[TestFixture]
	public class UnitTest
	{
        /// <summary>
        /// Constructor for unit test class.
        /// </summary>
		public UnitTest()
		{
			//
			// TODO: Add constructor logic here
			//
		}
        /// <summary>
        /// Test set up method.
        /// </summary>
        [SetUp] public void Init() 
        {	
            Console.WriteLine("********* New Test-case: *******");
            Console.WriteLine("photo_editor");
            TestImage = new Photo_Editor_Application.Picture();
        }
		
        private Photo_Editor_Application.Picture TestImage;
        private bool amIPassed = false;
        /// <summary>
        /// Tear down method for test case.
        /// </summary>
        [TearDown] public void Destroy()
        {
            TestImage.LoadedImage.Dispose();
            if(amIPassed)
            {	
                Console.WriteLine("=> PASSED");
            }
            else
            {
                Console.WriteLine("%%%%%% Failed");	
            }
        }
        /// <summary>
        /// Test for F:image_load_and_save.
        /// Windows standard dialogs are used, so really
        /// this tests only whether the default image loaded has the 
        /// dimensions
        /// that are expected!
        /// </summary>
        /// <requirement>F:image_load_and_save</requirement>
        [Test] public void LoadandSaveImageTest() 
        {
            const string fileName = "Test.jpg";	
			
            Console.WriteLine("image_load_and_save");
			
            Assertion.AssertEquals("Load Image, width", 
                TestImage.LoadedImage.Width, 2048);
            Assertion.AssertEquals("Load Image, height", 
                TestImage.LoadedImage.Height, 1536);
			
            TestImage.CropImage(200, 400);
            TestImage.SaveImage(fileName);

            TestImage.LoadImage("Hawaii.jpg");

            TestImage.LoadImage(fileName);

            Assertion.AssertEquals("Load Image, width", 
                TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Load Image, height", 
                TestImage.LoadedImage.Height, 400);
            amIPassed = true;
        }
        #region Do It Yourself Chapter 5
        /// <summary>
        /// Test for F:image_crop.
        /// Crops the default image and sees if dimensions are 
        /// changing accordingly!
        /// </summary>
        /// <requirement>F:image_crop</requirement>
        [Test] public void ImageCropTest()
        {
            Console.WriteLine("image_crop");

            TestImage.CropImage(200, 400);
            Assertion.AssertEquals("Crop Image, width", TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Crop Image, height", TestImage.LoadedImage.Height, 400);
            TestImage.CropImage(125, 238);
            Assertion.AssertEquals("Crop Image, width", TestImage.LoadedImage.Width, 125);
            Assertion.AssertEquals("Crop Image, height", TestImage.LoadedImage.Height, 238);

            TestImage.CropImage(200, 400);
            Assertion.AssertEquals("Crop Image, width", TestImage.LoadedImage.Width, 125);
            Assertion.AssertEquals("Crop Image, height", TestImage.LoadedImage.Height, 238);

            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_rotate.
        /// Tests different rotations by
        /// verifying the dimensions after rotation.
        /// </summary>
        /// <requirement>F:image_rotate</requirement>
        [Test] public void ImageRotateTest()
        {
            Console.WriteLine("image_rotate");
			
            TestImage.CropImage(200, 400);
            TestImage.RotateFlipImage(RotateFlipType.Rotate90FlipNone);
			
            Assertion.AssertEquals("Rotate Image 1, width", TestImage.LoadedImage.Width, 400);
            Assertion.AssertEquals("Rotate Image 1, height", TestImage.LoadedImage.Height, 200);

            TestImage.RotateFlipImage(RotateFlipType.Rotate90FlipNone);

            Assertion.AssertEquals("Rotate Image 2, width", TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Rotate Image 2, height", TestImage.LoadedImage.Height, 400);

            TestImage.RotateFlipImage(RotateFlipType.Rotate180FlipNone);

            Assertion.AssertEquals("Rotate Image 3, width", TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Rotate Image 3, height", TestImage.LoadedImage.Height, 400);

            amIPassed = true;
        }
		
        /// <summary>
        /// Test for F:image_flip.
        /// Flips the image and checks if the dimensions 
        /// are changing accordingly.
        /// </summary>
        /// <requirement>F:image_flip</requirement>
        [Test] public void ImageFlipTest()
        {
            Console.WriteLine("image_flip");
			
            TestImage.CropImage(200, 400);
            TestImage.RotateFlipImage(RotateFlipType.RotateNoneFlipX);
			
            Assertion.AssertEquals("Flip Image 1, width", TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Flip Image 1, height", TestImage.LoadedImage.Height, 400);

            TestImage.RotateFlipImage(RotateFlipType.RotateNoneFlipY);

            Assertion.AssertEquals("Flip Image 2, width", TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Flip Image 2, height", TestImage.LoadedImage.Height, 400);

            TestImage.RotateFlipImage(RotateFlipType.RotateNoneFlipXY);

            Assertion.AssertEquals("Flip Image 3, width", TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Flip Image 3, height", TestImage.LoadedImage.Height, 400);

            amIPassed = true;
        }
        #endregion
    }
}


